#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
#    Name:	Follow Me Helix
#      By:	sdmitch
#   Usage:	creates a helix around a selected centerline.  
#    Note:	This is a disk and resource hog so be very careful not to make the CL to long!	
#						Test on a short centerline to verify input parameters.
#    Date:	Sep 2011
#------------------------------------------------------------------------------------------------
require 'sketchup'
#
filename=File.basename(__FILE__)
#
if not file_loaded?(filename)
    UI.menu('Plugins').add_item('Follow Me Helix1') { SDM_HM.helix_maker1 }
end
#
file_loaded(filename)
#
module SDM_HM

	def self.helix_maker1
		sel=Sketchup.active_model.selection
		if (sel.length == 0) then
			UI.messagebox "Select the lines and/or curves that will be the shape and length of the helix you want to create"
			return
		else
			@radius=1.feet if !@radius;@dist_inc=1.inch if !@dist_inc;@ang_inc=30 if !@ang_inc
			ans=UI.inputbox(["Radius of helix:","linear Increment:","Angle Increment:"],[@radius,@dist_inc,@ang_inc],"Helix Generator")
			if !ans then;return;end
			@radius=ans[0].to_l;@dist_inc = ans[1].to_l; @ang_inc=ans[2].to_i
			verts = self.sort_edges(sel,2); last = verts.length - 1; min_length=1e6;
			for i in 1..last
				seg_length=verts[i-1].distance(verts[i])
				min_length=[min_length,seg_length].min
			end
			max_increment=min_length/(360/@ang_inc)
			if @dist_inc > max_increment
				if(UI.messagebox("Linear increment may be to big, #{max_increment.to_l} or less is recommended.  Accept?" ,MB_YESNO)==6)
					@dist_inc = max_increment.to_l
				end
			end
			Sketchup.active_model.start_operation("Helix")
			start_dist = 0; starting_angle = 0; pts = []; last_vector=nil
			1.upto(last) do |i|
				pt0=verts[i-1];angle = starting_angle; 
				next_vector=verts[i-1].vector_to(verts[i])
				seg_length=verts[i-1].distance(verts[i])
				if i> 1 #&& start_dist < seg_length
					ang=last_vector.angle_between(next_vector)
					num = 2; num=(ang/10.degrees).to_i + 1 if ang > 20.degrees; chg = 1.0/num ; 
					chg.step(1.0,chg) do |pct|
						vec = Geom::Vector3d.linear_combination(1.0-pct,last_vector,pct,next_vector)
						xaxis,yaxis,zaxis=vec.axes;xaxis.length = @radius; yaxis.length = @radius
						cosa = Math.cos(angle.degrees); sina = Math.sin(angle.degrees)
						vec = Geom::Vector3d.linear_combination(cosa,xaxis,sina,yaxis)
						pts.push pt0+vec;angle += @ang_inc; angle %= 360
					end
				end
				xaxis,yaxis,zaxis=next_vector.axes; dist = start_dist
				xaxis.length = @radius; yaxis.length = @radius
				while dist < seg_length
					cosa = Math.cos(angle.degrees); sina = Math.sin(angle.degrees)
					vec = Geom::Vector3d.linear_combination(cosa,xaxis,sina,yaxis)
					pts.push pt0.offset(next_vector,dist)+vec
					dist += @dist_inc;
					angle += @ang_inc; angle = 0 if angle >= 360
				end
				starting_angle = angle; start_dist = dist - seg_length; last_vector = next_vector
			end
			Sketchup.set_status_text "Adding #{pts.length} point helix curve to model.  This might take awhile."
			curve=Sketchup.active_model.entities.add_curve(pts)
			Sketchup.active_model.commit_operation
		end
	end

	def self.sort_edges(sel,flag)
		edges=sel.select {|x| x.is_a? Sketchup::Edge}
	#	---------------- This code copied from FAK.rb ------------------
		return if edges.length==0
		re=[];rp=[]	# split the output into two arrays
		while edges.length!=0
			start=nil
			edges.each{|x|
			start=x.start; break  if (x.start.edges&edges).length==1
			start=x.end; break if (x.end.edges&edges).length==1
			}
			vts=[]; curve=[]
			while start
				next_edge=(start.edges&edges-curve)[0]
				if next_edge
					curve << next_edge
					vts << start
					start=next_edge.other_vertex(start)
				else
					vts << start
					start=nil
				end
			end
			edges-=curve
			re += curve 
			rp += vts 
		end
		rp.collect!{|x| x.position}
		return rp if flag==2
		return re if flag==1
		return re,rp	#re=edges, rp=points
	end
	
end
